package ca.recrown.islandsurvivalcraft.interaction.commands.runnables;

import java.util.HashSet;
import java.util.LinkedList;
import java.util.concurrent.CompletableFuture;

import org.bukkit.Bukkit;
import org.bukkit.Location;
import org.bukkit.Particle;
import org.bukkit.World;
import org.bukkit.command.CommandSender;
import org.bukkit.entity.Player;

import ca.recrown.islandsurvivalcraft.IslandSurvivalCraft;
import ca.recrown.islandsurvivalcraft.world.WorldInfo;

public class HighlightIslandCommand implements CommandRunnable {
    private final int CHECK_RADIUS = 3;
    private final int PARTICLE_AMOUNT = 2;
    private final Particle PARTICLE_TYPE = Particle.FIREWORKS_SPARK;
    private final double OFFSET_X = 0.0d, OFFSET_Y = 0.0d, OFFSET_Z = 0.0d;

    HashSet<Player> playersHighlighting = new HashSet<>();
    LinkedList<Player> waitingList = new LinkedList<>();

    @Override
    public String getDescription() {
        return "Displays particle effects at edge of islands.";
    }

    @Override
    public CompletableFuture<String> invoke(CommandSender sender, String[] args) {
        CompletableFuture<String> completableFuture = new CompletableFuture<>();
        if (!(sender instanceof Player)) {
            completableFuture.complete("This command can only be run as a player!");
            return completableFuture;
        }
        Player player = (Player) sender;
        if (args[0].isEmpty()) {
            completableFuture.complete("Arguments required. Please refer to \"/IslandSurvivalCraft help\" for more info.");
            return completableFuture;
        }
        if (args[0].toLowerCase().equals("start")) {
            if (playersHighlighting.add(player)) {
                waitingList.add(player);
                completableFuture.complete("You are now highlighting islands.");
            } else {
                completableFuture.complete("You are already highlighting islands.");
            }
        } else if (args[0].toLowerCase().equals("stop")) {
            if (playersHighlighting.remove(player)) {
                completableFuture.complete("You are no longer highlighting islands.");
            } else {
                completableFuture.complete("You weren't highlighting islands.");
            }
        } else {
            completableFuture.complete(String.format("Argument \"%d\" was not understood. Please refer to \"/IslandSurvivalCraft help\" for more info.", args[0]));
        }
        return completableFuture;
    }

    @Override
    public void initialize(IslandSurvivalCraft islandsurvivalcraft) {
        Bukkit.getScheduler().scheduleSyncRepeatingTask(islandsurvivalcraft, new Runnable() {
            @Override
            public void run() {
                Player player = waitingList.poll();
                if (player == null || !player.isOnline() || !playersHighlighting.contains(player)) {
                    playersHighlighting.remove(player);
                    return;
                }
                World world = player.getWorld();
                WorldInfo worldInfo = islandsurvivalcraft.getWorldInfoManager().retrieve(world);
                Location playerLocation = player.getLocation();
                int playerX = playerLocation.getBlockX(), playerY = playerLocation.getBlockY(), playerZ = playerLocation.getBlockZ();
                for (int x = 1; x < CHECK_RADIUS; x++) {
                    for (int z = 1; z < CHECK_RADIUS; z++) {
                        if (worldInfo.getIslandMap().isIsland(playerX + x, playerZ + z)) {
                            world.spawnParticle(PARTICLE_TYPE, x + playerX, playerY, z + playerZ, PARTICLE_AMOUNT, OFFSET_X, OFFSET_Y, OFFSET_Z);
                        }
                        if (worldInfo.getIslandMap().isIsland(playerX - x, playerZ - z)) {
                            world.spawnParticle(PARTICLE_TYPE, playerX - x, playerY, playerZ - z, PARTICLE_AMOUNT, OFFSET_X, OFFSET_Y, OFFSET_Z);
                        }
                    }
                }
                waitingList.add(player);
            }
        }, 0, 5);

    }

    @Override
    public String getDetailedDescription() {
        return "A helpful debugging command to analyze what the plugin considers an island by displaying particle effects around the player that is a part of the island. The <start | stop> argument is required to dicate whether or not to start the highlighting or stop it.";
    }
    
}